use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007004
use Test::Spelling 0.12;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib ) );
__DATA__
Abstract
Alexey
Anderson
Category
Change
Changelog
Class
Comment
Component
Factory
FactoryTypes
Issue
Item
Iterator
JIRA
LinkType
Melezhik
Mixins
Packy
Project
Query
REST
Sprint
Status
TimeTracking
Transition
Transitions
Type
User
Version
Worklog
lib
melezhik
packy
